library nexstate_flutter;

import 'package:flutter/material.dart';
import 'package:nexstate/nexstate.dart';

/// Wrap this widget around parts that need to be rebuilt every time a specific
/// nexstate store is updated with `setState` method.
class Consumer<T extends Store> extends StatefulWidget {
  final T store;
  final Widget Function(BuildContext context, T store) builder;

  const Consumer({
    required this.store,
    required this.builder,
    Key? key,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() => _ConsumerState<T>();
}

class _ConsumerState<T extends Store> extends State<Consumer<T>> {
  late final Subscription subscription;

  @override
  void initState() {
    super.initState();
    subscription = widget.store.subscribe(() => setState(() {}));
  }

  @override
  void dispose() {
    subscription.cancel();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return widget.builder(context, widget.store);
  }
}

/// Extend this class to create providers. You can assign nexstate stores as
/// fields to providers. This way, stores can be used deep in your widget tree.
abstract class ProviderScaffold extends InheritedWidget {
  const ProviderScaffold({
    required Widget child,
    Key? key,
  }) : super(key: key, child: child);

  static T of<T extends ProviderScaffold>(BuildContext context) {
    final result = context.dependOnInheritedWidgetOfExactType<T>();
    assert(result != null, 'No $T found in context');
    return result!;
  }

  @override
  bool updateShouldNotify(covariant InheritedWidget oldWidget) =>
      oldWidget != this;
}
